/*
 * Copyright (C) 2001-2006 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */
package mir.producer.reader;

import java.util.HashMap;
import java.util.Map;

public class ProducerNodeBuilderLibrary {
  private Map nodeBuilders;

  public ProducerNodeBuilderLibrary() {
    nodeBuilders = new HashMap();
  }

  /**
   * Register a builder
   * @param aName
   * @param aProducerNodeBuilderClass
   * @throws ProducerConfigExc
   * @throws ProducerConfigFailure
   */
  public void registerBuilder(String aName, Class aProducerNodeBuilderClass) throws ProducerConfigExc, ProducerConfigFailure {
    try {
      registerFactory(aName, new ProducerNodeBuilder.DefaultProducerNodeBuilderFactory( aProducerNodeBuilderClass ));
    }
    catch (Throwable t) {
      throw new ProducerConfigFailure("Error while registering builder '"+aName+"': "+t.getMessage(), t);
    }
  }

  /**
   * Register a  {@link ProducerNodeBuilder.ProducerNodeBuilderFactory} for {@link mir.producer.ProducerNode}s
   * @param aTag the tag to bind this factory to: every producers.xml tag with the name <code>aTag</code>
   *     will be directed to this factory.
   *
   * @param aFactory the factory to bind the tag to.
   */
  public void registerFactory(String aTag, ProducerNodeBuilder.ProducerNodeBuilderFactory aFactory) {
    nodeBuilders.put(aTag, aFactory);
  }

  public boolean hasBuilderForName(String aName) {
    return nodeBuilders.containsKey(aName);
  }

  public ProducerNodeBuilder constructBuilder(String aName) throws ProducerConfigExc {
    if (hasBuilderForName(aName))
      return ((ProducerNodeBuilder.ProducerNodeBuilderFactory) nodeBuilders.get(aName)).makeBuilder();
		throw new ProducerConfigExc("ProducerNodeBuilder: no builder with name '" + aName + "' found.");
  }
}
